#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
import json

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_k8s_auth_role
version_added: "4.3.1"
short_description: Hashicorp Vault kubernetes secret engine role
description:
    - Module to define a kubernetes role that vault can generate dynamic credentials for vault
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: kubernetes
    name:
        description:
            - name of the role in vault
    ttl:
        description:
            - TTL period of tokens issued using this role in seconds
    max_ttl:
        description:
            - maximum allowed lifetime of tokens issued in seconds using this role.
    policies:
        description:
            - policies to be set on tokens issued using this role.
    period:
        description:
            - If set, indicates that the token generated using this role should never expire. The token should
              be renewed within the duration specified by this value. At each renewal, the token's TTL will be set
              to the value of this parameter
    role_file:
        description:
            - File with a json object containing play parameters. pass all params but name, state, mount_point which
              stay in the ansible play
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_k8s_auth_role:
        name: "test"
        policies: ["test", "test2"]
        bound_service_account_names: ["vault-auth"]
        bound_service_account_namespaces: ["default", "some-app"]

    - hashivault_k8s_auth_role:
        name: test
        role_file: some_k8s_role.json
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['bound_service_account_names'] = dict(required=False, type='list', default=[])
    argspec['bound_service_account_namespaces'] = dict(required=False, type='list', default=[])
    argspec['ttl'] = dict(required=False, type='int', default=0)
    argspec['max_ttl'] = dict(required=False, type='int', default=0)
    argspec['policies'] = dict(required=False, type='list')
    argspec['period'] = dict(required=False, type='int', default=0)
    argspec['mount_point'] = dict(required=False, type='str', default='kubernetes')
    argspec['role_file'] = dict(required=False, type='str')
    argspec['state'] = dict(required=False, type='str', default='present', choices=['present', 'absent'])

    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_k8s_auth_role(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_k8s_auth_role(module):
    params = module.params
    client = hashivault_auth_client(params)
    mount_point = params.get('mount_point').strip('/')
    role_file = params.get('role_file')
    name = params.get('name').strip('/')
    state = params.get('state')
    desired_state = dict()
    exists = False
    changed = False

    if role_file:
        desired_state = json.loads(open(params.get('role_file'), 'r').read())
    else:
        desired_state['bound_service_account_names'] = params.get('bound_service_account_names')
        desired_state['bound_service_account_namespaces'] = params.get('bound_service_account_namespaces')
        desired_state['ttl'] = params.get('ttl')
        desired_state['max_ttl'] = params.get('max_ttl')
        desired_state['period'] = params.get('period')
        desired_state['policies'] = params.get('policies')

    # check if role exists
    try:
        existing_roles = client.auth.kubernetes.list_roles(mount_point=mount_point)
        if name in existing_roles['keys']:
            # this role exists
            exists = True
    except Exception:
        # no roles exist yet
        pass

    if not exists and state == 'present':
        changed = True

    # compare current_state to desired_state
    if exists and state == 'present' and not changed:
        current_state = client.auth.kubernetes.read_role(name=name, mount_point=mount_point)
        # Map a couple elements
        if 'ttl' not in current_state and 'token_ttl' in current_state:
            current_state['ttl'] = current_state['token_ttl']
        if 'max_ttl' not in current_state and 'token_max_ttl' in current_state:
            current_state['max_ttl'] = current_state['token_max_ttl']
        if 'period' not in current_state and 'token_period' in current_state:
            current_state['period'] = current_state['token_period']
        for key in desired_state.keys():
            if key in current_state and desired_state[key] != current_state[key]:
                changed = True
    elif exists and state == 'absent':
        changed = True

    if changed and state == 'present' and not module.check_mode:
        client.auth.kubernetes.create_role(name=name, mount_point=mount_point, **desired_state)

    elif changed and state == 'absent' and not module.check_mode:
        client.auth.kubernetes.delete_role(name=name, mount_point=mount_point)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import compare_state
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_url
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Set URLs
description:
    - This module allows setting the issuing certificate endpoints, CRL distribution points, and OCSP server endpoints
      that will be encoded into issued certificates.
    - You can update any of the values at any time without affecting the other existing values.
    - To remove the values, simply use a blank string as the parameter.
options:
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
    issuing_certificates:
        type: list
        description:
            - Specifies the URL values for the Issuing Certificate field.
    crl_distribution_points:
        type: list
        description:
            -  Specifies the URL values for the CRL Distribution Points field.
    ocsp_servers:
        type: list
        description:
            -  Specifies the URL values for the OCSP Servers field.
extends_documentation_fragment:
    - hashivault
'''

EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_url:
        issuing_certificates:
            - 'http://127.0.0.1:8200/v1/pki/ca'
        crl_distribution_points:
            - 'http://127.0.0.1:8200/v1/pki/crl'

'''


def main():
    argspec = hashivault_argspec()
    argspec['issuing_certificates'] = dict(required=False, type='list', default=[])
    argspec['crl_distribution_points'] = dict(required=False, type='list', default=[])
    argspec['ocsp_servers'] = dict(required=False, type='list', default=[])
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    supports_check_mode = True

    module = hashivault_init(argspec, supports_check_mode)
    result = hashivault_pki_url(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_url(module):
    params = module.params
    client = hashivault_auth_client(params)

    mount_point = params.get('mount_point').strip('/')

    desired_state = {
        'issuing_certificates': params.get('issuing_certificates'),
        'crl_distribution_points': params.get('crl_distribution_points'),
        'ocsp_servers': params.get('ocsp_servers')
    }

    # check if engine is enabled
    changed, err = check_secrets_engines(module, client)
    if err:
        return err

    # check if config exists
    current_state = {}
    try:
        current_state = client.secrets.pki.read_urls(mount_point=mount_point).get('data')
    except Exception:
        # not configured yet.
        changed = True

    # compare current_state to desired_state
    if not changed:
        changed = not compare_state(desired_state, current_state)

    # make the changes!
    if changed and not module.check_mode:
        client.secrets.pki.set_urls(mount_point=mount_point, params=desired_state)
    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import compare_state
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_crl
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Set CRL Configuration
description:
    - This module allows setting the duration for which the generated CRL should be marked valid.
    - If the CRL is disabled, it will return a signed but zero-length CRL for any request.
    - If enabled, it will re-build the CRL.
options:
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
    expiry:
        required: true
        type: str
        description:
            - Specifies the time until expiration.
    disable:
        type: bool
        default: false
        description:
            - Disables or enables CRL building.
extends_documentation_fragment:
    - hashivault
'''
EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_crl:
        expiry: 72h
        disable: false

'''


def main():
    argspec = hashivault_argspec()
    argspec['expiry'] = dict(required=True, type='str')
    argspec['disable'] = dict(required=False, type='bool', default=False)
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    supports_check_mode = True

    module = hashivault_init(argspec, supports_check_mode)
    result = hashivault_pki_crl(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_crl(module):
    params = module.params
    client = hashivault_auth_client(params)

    mount_point = params.get('mount_point').strip('/')

    desired_state = {
        'disable': params.get('disable'),
        'expiry': params.get('expiry')
    }

    # check if engine is enabled
    changed, err = check_secrets_engines(module, client)
    if err:
        return err

    # compare current_state to desired_state
    if not changed:
        from hvac.exceptions import InvalidPath
        try:
            current_state = client.secrets.pki.read_crl_configuration(mount_point=mount_point).get('data')
            changed = not compare_state(desired_state, current_state)
        except InvalidPath:
            changed = True

    # make the changes!
    if changed and not module.check_mode:
        client.secrets.pki.set_crl_configuration(mount_point=mount_point, extra_params=desired_state)
    return {'changed': changed}


if __name__ == '__main__':
    main()



import os
import warnings

import hvac
import requests
from ansible.module_utils.basic import AnsibleModule
from hvac.exceptions import InvalidPath


def hashivault_argspec():
    argument_spec = dict(
        url=dict(required=False, default=os.environ.get('VAULT_ADDR', ''), type='str'),
        ca_cert=dict(required=False, default=os.environ.get('VAULT_CACERT', ''), type='str'),
        ca_path=dict(required=False, default=os.environ.get('VAULT_CAPATH', ''), type='str'),
        client_cert=dict(required=False, default=os.environ.get('VAULT_CLIENT_CERT', ''), type='str'),
        client_key=dict(required=False, default=os.environ.get('VAULT_CLIENT_KEY', ''), type='str'),
        verify=dict(required=False, default=(not os.environ.get('VAULT_SKIP_VERIFY', '')), type='bool'),
        authtype=dict(required=False, default=os.environ.get('VAULT_AUTHTYPE', 'token'), type='str'),
        login_mount_point=dict(required=False, default=os.environ.get('VAULT_LOGIN_MOUNT_POINT', None), type='str'),
        token=dict(required=False, default=hashivault_default_token(), type='str', no_log=True),
        username=dict(required=False, default=os.environ.get('VAULT_USER', ''), type='str'),
        password=dict(required=False, default=os.environ.get('VAULT_PASSWORD', ''), type='str', no_log=True),
        role_id=dict(required=False, default=os.environ.get('VAULT_ROLE_ID', ''), type='str', no_log=True),
        secret_id=dict(required=False, default=os.environ.get('VAULT_SECRET_ID', ''), type='str', no_log=True),
        aws_header=dict(required=False, default=os.environ.get('VAULT_AWS_HEADER', ''), type='str', no_log=True),
        namespace=dict(required=False, default=os.environ.get('VAULT_NAMESPACE', None), type='str')
    )
    return argument_spec


def hashivault_init(argument_spec, supports_check_mode=False, required_if=None, required_together=None,
                    required_one_of=None, mutually_exclusive=None):
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=supports_check_mode,
                           required_if=required_if, required_together=required_together,
                           required_one_of=required_one_of, mutually_exclusive=mutually_exclusive)
    module.no_log_values.discard("0")
    module.no_log_values.discard(0)
    module.no_log_values.discard("1")
    module.no_log_values.discard(1)
    module.no_log_values.discard(True)
    module.no_log_values.discard(False)
    return module


def get_ec2_iam_role():
    request = requests.get(url='http://169.254.169.254/latest/meta-data/iam/security-credentials/')
    request.raise_for_status()
    return request.content


def get_ec2_iam_credentials(header_value, role_id):
    role_name = get_ec2_iam_role()
    metadata_url = 'http://169.254.169.254/latest/meta-data/iam/security-credentials/{role}'.format(
        role=role_name
    )
    response = requests.get(url=metadata_url)
    response.raise_for_status()
    credentials = response.json()
    return dict(
        access_key=credentials['AccessKeyId'],
        secret_key=credentials['SecretAccessKey'],
        session_token=credentials['Token'],
        header_value=header_value,
        role=role_id
    )


def hashivault_client(params):
    url = params.get('url')
    ca_cert = params.get('ca_cert')
    ca_path = params.get('ca_path')
    client_cert = params.get('client_cert')
    client_key = params.get('client_key')
    cert = (client_cert, client_key)
    check_verify = params.get('verify')
    namespace = params.get('namespace', None)
    if check_verify == '' or check_verify:
        if ca_cert:
            verify = ca_cert
        elif ca_path:
            verify = ca_path
        else:
            verify = check_verify
    else:
        verify = check_verify
    client = hvac.Client(url=url, cert=cert, verify=verify, namespace=namespace)
    return client


def hashivault_auth(client, params):
    token = params.get('token')
    authtype = params.get('authtype')
    login_mount_point = params.get('login_mount_point', authtype)
    if not login_mount_point:
        login_mount_point = authtype
    username = params.get('username')
    password = params.get('password')
    secret_id = params.get('secret_id')
    role_id = params.get('role_id')

    if authtype == 'github':
        client.auth.github.login(token, mount_point=login_mount_point)
    elif authtype == 'userpass':
        client.auth_userpass(username, password, mount_point=login_mount_point)
    elif authtype == 'ldap':
        client.auth.ldap.login(username, password, mount_point=login_mount_point)
    elif authtype == 'approle':
        client = AppRoleClient(client, role_id, secret_id, mount_point=login_mount_point)
    elif authtype == 'tls':
        client.auth_tls()
    elif authtype == 'aws':
        credentials = get_ec2_iam_credentials(params.get['aws_header'], role_id)
        client.auth_aws_iam(**credentials)
    else:
        client.token = token
    return client


def hashivault_auth_client(params):
    client = hashivault_client(params)
    return hashivault_auth(client, params)


def hashiwrapper(function):
    def wrapper(*args, **kwargs):
        result = {"changed": False, "rc": 0}
        result.update(function(*args, **kwargs))
        return result
    return wrapper


def hashivault_default_token():
    """Get a default Vault token from an environment variable or a file."""
    if 'VAULT_TOKEN' in os.environ:
        return os.environ['VAULT_TOKEN']
    token_file = os.path.expanduser('~/.vault-token')
    if os.path.exists(token_file):
        with open(token_file, 'r') as f:
            return f.read().strip()
    return ''


@hashiwrapper
def hashivault_read(params):
    result = {"changed": False, "rc": 0}
    client = hashivault_auth_client(params)
    version = params.get('version')
    mount_point = params.get('mount_point')
    secret = params.get('secret')

    key = params.get('key')
    default = params.get('default')
    if secret.startswith('/'):
        secret = secret.lstrip('/')
        mount_point = ''
    if mount_point:
        secret_path = '%s/%s' % (mount_point, secret)
    else:
        secret_path = secret

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        try:
            if version == 2:
                response = client.secrets.kv.v2.read_secret_version(secret, mount_point=mount_point)
            else:
                response = client.read(secret_path)
        except InvalidPath:
            response = None
        except Exception as e:
            result['rc'] = 1
            result['failed'] = True
            error_string = "%s(%s)" % (e.__class__.__name__, e)
            result['msg'] = u"Error %s reading %s" % (error_string, secret_path)
            return result
        if not response:
            if default is not None:
                result['value'] = default
                return result
            result['rc'] = 1
            result['failed'] = True
            result['msg'] = u"Secret %s is not in vault" % secret_path
            return result
        if version == 2:
            try:
                data = response.get('data', {})
                data = data.get('data', {})
            except Exception:
                data = str(response)
        else:
            data = response['data']
        lease_duration = response.get('lease_duration', None)
        if lease_duration is not None:
            result['lease_duration'] = lease_duration
        lease_id = response.get('lease_id', None)
        if lease_id is not None:
            result['lease_id'] = lease_id
        renewable = response.get('renewable', None)
        if renewable is not None:
            result['renewable'] = renewable
        wrap_info = response.get('wrap_info', None)
        if wrap_info is not None:
            result['wrap_info'] = wrap_info
    if key and key not in data:
        if default is not None:
            result['value'] = default
            return result
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Key %s is not in secret %s" % (key, secret_path)
        return result
    if key:
        value = data[key]
    else:
        value = data
    result['value'] = value
    return result


class AppRoleClient(object):
    """
    hvac.Client decorator which generates and sets a new approle token on every
    function call. This allows multiple calls to Vault without having to manually
    generate and set a token on every Vault call.
    """

    def __init__(self, client, role_id, secret_id, mount_point):
        object.__setattr__(self, 'client', client)
        object.__setattr__(self, 'role_id', role_id)
        object.__setattr__(self, 'secret_id', secret_id)
        object.__setattr__(self, 'login_mount_point', mount_point)

    def __setattr__(self, name, val):
        """
        sets attribute in decorated class (Client)
        """
        client = object.__getattribute__(self, 'client')
        client.__setattr__(name, val)

    def __getattribute__(self, name):
        """
        generates and sets new approle token in decorated class (Client)
        returns decorated class (Client) attribute
        """
        client = object.__getattribute__(self, 'client')
        attr = client.__getattribute__(name)

        role_id = object.__getattribute__(self, 'role_id')
        secret_id = object.__getattribute__(self, 'secret_id')
        login_mount_point = object.__getattribute__(self, 'login_mount_point')
        resp = client.auth_approle(role_id, secret_id=secret_id, mount_point=login_mount_point)
        client.token = str(resp['auth']['client_token'])
        return attr


def check_secrets_engines(module, client):
    """Checks if secrets engine is mounted

    :param module: Ansible module. Must contain mount_point in parameters.
    :param mounted: HVAC client
    :return: change status, error
    :rtype: (bool, dict)
    """
    changed = False
    err = None
    try:
        if (module.params.get('mount_point') + "/") not in client.sys.list_mounted_secrets_engines()['data'].keys():
            err = {'failed': True, 'msg': 'secret engine is not enabled', 'rc': 1}
    except Exception:
        if module.check_mode:
            changed = True
        else:
            err = {'failed': True, 'msg': 'secret engine is not enabled or namespace does not exist', 'rc': 1}
    return changed, err


def check_auth_methods(module, client):
    """Checks if auth engine is mounted

    :param module: Ansible module. Must contain mount_point in parameters.
    :param mounted: HVAC client
    :return: change status, error
    :rtype: (bool, dict)
    """
    changed = False
    err = None
    try:
        if (module.params.get('mount_point') + "/") not in client.sys.list_auth_methods()['data'].keys():
            err = {'failed': True, 'msg': 'auth method is not enabled', 'rc': 1}
    except Exception:
        if module.check_mode:
            changed = True
        else:
            err = {'failed': True, 'msg': 'auth mount is not enabled or namespace does not exist', 'rc': 1}

    return changed, err


def check_pki_role(name, mount_point, client):
    """Checks if role is prtesent in secrets engine

    :param module: Ansible module. Must contain mount_point in parameters.
    :param mounted: HVAC client
    :return: change status, error
    :rtype: (bool, dict)
    """
    try:
        return client.secrets.pki.read_role(name=name, mount_point=mount_point).get('data')
    except Exception:
        return None


def compare_state(desired_state, current_state, ignore=None):
    """Compares desired state to current state. Returns true if objects are equal

    Recursively walks dict object to compare all keys

    :param desired_state: The state user desires.
    :param current_state: The state that currently exists.
    :param ignore: Ignore these keys.
    :type ignore: list

    :return: True if the states are the same.
    :rtype: bool
    """

    if ignore is None:
        ignore = []
    if (type(desired_state) is list):
        if ((type(current_state) != list) or (len(desired_state) != len(current_state))):
            return False
        return set(desired_state) == set(current_state)

    if (type(desired_state) is dict):
        if (type(current_state) != dict):
            return False

        # iterate over dictionary keys
        for key in desired_state.keys():
            if key in ignore:
                continue
            v = desired_state[key]
            if ((key not in current_state) or (not compare_state(v, current_state.get(key)))):
                return False
        return True

    # lots of things get handled as strings in ansible that aren't necessarily strings, can extend this list later.
    if isinstance(desired_state, str) and isinstance(current_state, int):
        current_state = str(current_state)

    return ((desired_state == current_state))


def get_keys_updated(desired_state, current_state, ignore=None):
    """Return list of keys that have different values

    Recursively walks dict object to compare all keys

    :param desired_state: The state user desires.
    :type desired_state: dict
    :param current_state: The state that currently exists.
    :type current_state: dict
    :param ignore: Ignore these keys.
    :type ignore: list

    :return: Different items
    :rtype: list
    """

    if ignore is None:
        ignore = []

    differences = []
    for key in desired_state.keys():
        if key in ignore:
            continue
        v = desired_state[key]
        if ((key not in current_state) or (not compare_state(v, current_state.get(key)))):
            differences.append(key)
    return differences



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

from hvac.exceptions import InvalidPath


ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_aws_auth_role_create
version_added: "4.4.8"
short_description: Hashicorp Vault aws auth configure module
description:
    - Module to configure an aws auth backend
options:
    max_retries:
        description:
            - Number of max retries the client should use for recoverable errors.
    access_key:
        description:
            - AWS Access key with permissions to query AWS APIs.
    secret_key:
        description:
            - AWS Secret key with permissions to query AWS APIs
    endpoint:
        description:
            - URL to override the default generated endpoint for making AWS EC2 API calls
    iam_endpoint:
        description:
            - URL to override the default generated endpoint for making AWS IAM API calls
    sts_endpoint:
        description:
            - URL to override the default generated endpoint for making AWS STS API calls
    iam_server_id_header_value:
        description:
            - The value to require in the X-Vault-AWS-IAM-Server-ID header
    mount_point:
        description:
            - location or "path" where this auth method is mounted
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_aws_auth_config:
        mount_point: myaws
        access_key: AwesomeAccessK3y
        secret_key: N0b0dyKnows
'''


def main():
    argspec = hashivault_argspec()
    argspec['max_retries'] = dict(required=False, type='int')
    argspec['access_key'] = dict(required=False, type='str')
    argspec['secret_key'] = dict(required=False, type='str')
    argspec['endpoint'] = dict(required=False, type='str')
    argspec['iam_endpoint'] = dict(required=False, type='str')
    argspec['sts_endpoint'] = dict(required=False, type='str')
    argspec['iam_server_id_header_value'] = dict(required=False, type='str')
    argspec['mount_point'] = dict(required=False, default='aws', type='str')
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    module = hashivault_init(argspec)
    result = hashivault_aws_auth_role_create(module.params)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_aws_auth_role_create(params):
    state = params.get('state')
    mount_point = params.get('mount_point').strip('/')
    desired_state = {
        'max_retries': params.get('max_retries'),
        'access_key': params.get('access_key'),
        'secret_key': params.get('secret_key'),
        'endpoint': params.get('endpoint'),
        'iam_endpoint': params.get('iam_endpoint'),
        'sts_endpoint': params.get('sts_endpoint'),
        'iam_server_id_header_value': params.get('iam_server_id_header_value'),
    }
    client = hashivault_auth_client(params)

    if state == 'present':
        client.auth.aws.configure(mount_point=mount_point, **desired_state)
        return {'changed': True}
    client.auth.aws.delete_config(mount_point)
    return {'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_approle_role_list
version_added: "3.8.0"
short_description: Hashicorp Vault approle list roles module
description:
    - Module to list approle roles from Hashicorp Vault.
options:
    mount_point:
        description:
            - mount point for role
        default: approle
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_approle_role_list:
      register: 'vault_approle_role_list'
    - debug: msg="Roles are {{vault_approle_role_list.roles}}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='approle')
    module = hashivault_init(argspec)
    result = hashivault_approle_role_list(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_approle_role_list(params):
    client = hashivault_auth_client(params)
    roles = client.list_roles(mount_point=params.get('mount_point'))
    roles = roles.get('data', {}).get('keys', [])
    return {'roles': roles}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

from hvac.exceptions import InvalidPath


ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_aws_auth_role
version_added: "4.4.8"
short_description: Hashicorp Vault aws auth create role module
description:
    - Module to create an aws auth role
options:
    name:
        description:
            - role name.
    auth_type:
        description:
            -  auth type permitted for this role. Valid choices are ec2 and iam
    bound_ami_id:
        description:
            - Defines a constraint on the EC2 instances that can perform the login operation that they should be using\
             the AMI ID specified
    bound_account_id:
        description:
            - Defines a constraint on the EC2 instances that can perform the login operation that they should be using\
             the account ID
    bound_region:
        description:
            - The bound region for the role
    bound_vpc_id:
        description:
            - Defines a constraint on the EC2 instances that can perform the login operation that they be associated\
             with the VPC ID that matches the value
    bound_subnet_id:
        description:
            - Defines a constraint on the EC2 instances that can perform the login operation that they be associated\
             with the subnet ID
    bound_iam_role_arn:
        description:
            - Defines a constraint on the EC2 instances that can perform the login operation that they must match the\
             IAM role ARN
    bound_iam_instance_profile_arn:
        description:
            - Defines a constraint on the EC2 instances that can perform the login operation that they must be\
             associated with an IAM instance profile
    bound_ec2_instance_id:
        description:
            -EC2 instance id
    role_tag:
        description:
            - Role tag
    bound_iam_principal_arn:
        description:
            - IAM principal arn
    inferred_entity_type:
        description:
            - Instructs Vault to turn on inferencing. The only valid value is ec2_instance
    resolve_aws_unique_ids:
        description:
            - If set to true, the bound_iam_principal_arn is resolved to an AWS Unique ID for the bound principal ARN.
    ttl:
        description:
            - The TTL period of tokens issued using this role, provided as a number of seconds.
    max_ttl:
        description:
            - The maximum allowed lifetime of tokens issued using this role, provided as a number of seconds
    period:
        description:
            - The period
    policies:
        description:
            - policies for the role.
    allow_instance_migration:
        description:
            - If set to true, allows migration of the underlying instance where the client resides.
    disallow_reauthentication:
        description:
            - If set to true, only allows a single token to be granted per instance ID.
    mount_point:
        description:
            - location where this auth_method will be mounted. also known as "path"
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_aws_auth_role_create:
        name: myrole
        auth_type: iam
        inferred_entity_type: ec2_instance
        inferred_aws_region: eu-west-1
        bound_iam_role_arn: arn:aws:iam::12345678:root/ec2-role
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['auth_type'] = dict(required=False, type='str')
    argspec['resolve_aws_unique_ids'] = dict(required=False, type='bool')
    argspec['bound_ami_id'] = dict(required=False, type='str')
    argspec['bound_account_id'] = dict(required=False, type='str')
    argspec['bound_region'] = dict(required=False, type='str')
    argspec['bound_vpc_id'] = dict(required=False, type='str')
    argspec['bound_subnet_id'] = dict(required=False, type='str')
    argspec['bound_iam_role_arn'] = dict(required=False, type='str')
    argspec['bound_iam_instance_profile_arn'] = dict(required=False, type='str')
    argspec['bound_ec2_instance_id'] = dict(required=False, type='str')
    argspec['role_tag'] = dict(required=False, type='str')
    argspec['bound_iam_principal_arn'] = dict(required=False, type='str')
    argspec['inferred_entity_type'] = dict(required=False, type='str')
    argspec['inferred_aws_region'] = dict(required=False, type='str')
    argspec['ttl'] = dict(required=False, type='int')
    argspec['max_ttl'] = dict(required=False, type='int')
    argspec['period'] = dict(required=False, type='int')
    argspec['policies'] = dict(required=False, type='list')
    argspec['allow_instance_migration'] = dict(required=False, type='bool')
    argspec['disallow_reauthentication'] = dict(required=False, type='bool')
    argspec['mount_point'] = dict(required=False, default='aws', type='str')
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    module = hashivault_init(argspec)
    result = hashivault_aws_auth_role_create(module.params)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_aws_auth_role_create(params):
    state = params.get('state')
    name = params.get('name').strip('/')
    mount_point = params.get('mount_point').strip('/')
    desired_state = {
        'auth_type': params.get('auth_type'),
        'resolve_aws_unique_ids': params.get('resolve_aws_unique_ids'),
        'bound_ami_id': params.get('bound_ami_id'),
        'bound_account_id': params.get('bound_account_id'),
        'bound_region': params.get('bound_region'),
        'bound_vpc_id': params.get('bound_vpc_id'),
        'bound_subnet_id': params.get('bound_subnet_id'),
        'bound_iam_role_arn': params.get('bound_iam_role_arn'),
        'bound_iam_instance_profile_arn': params.get('bound_iam_instance_profile_arn'),
        'bound_ec2_instance_id': params.get('bound_ec2_instance_id'),
        'role_tag': params.get('role_tag'),
        'bound_iam_principal_arn': params.get('bound_iam_principal_arn'),
        'inferred_entity_type': params.get('inferred_entity_type'),
        'inferred_aws_region': params.get('inferred_aws_region'),
        'ttl': params.get('ttl'),
        'max_ttl': params.get('max_ttl'),
        'period': params.get('period'),
        'policies': params.get('policies'),
        'allow_instance_migration': params.get('allow_instance_migration'),
        'disallow_reauthentication': params.get('disallow_reauthentication'),
    }
    client = hashivault_auth_client(params)
    if not name:
        name = mount_point

    if state == 'present':
        client.auth.aws.create_role(name, mount_point=mount_point, **desired_state)
        return {'changed': True}
    client.auth.aws.delete_role(name, mount_point)
    return {'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
import requests

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_oidc_auth_method_config
version_added: "4.1.1"
short_description: Hashicorp Vault OIDC auth method config
description:
    - Module to configure an OIDC auth mount
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: oidc
    default_role:
        description:
            - The default role to use if none is provided during login.
    oidc_discovery_url:
        description:
            - The OIDC Discovery URL, without any .well-known component (base path). Cannot be used with "jwks_url" or
              "jwt_validation_pubkeys".
    oidc_client_id:
        description:
            - The OAuth Client ID from the provider for OIDC roles.
    oidc_client_secret:
        description:
            - The OAuth Client Secret from the provider for OIDC roles.
    bound_issuer:
        description:
            - The value against which to match the iss claim in a JWT.
    jwks_ca_pem:
        description:
            - The CA certificate or chain of certificates, in PEM format, to use to validate connections to the JWKS
              URL. If not set, system certificates are used.
    jwks_url:
        description:
            - JWKS URL to use to authenticate signatures. Cannot be used with "oidc_discovery_url" or
              "jwt_validation_pubkeys".
    jwt_supported_algs:
        description:
            - A list of supported signing algorithms.
        default: RS256
    jwt_validation_pubkeys:
        description:
            - A list of PEM-encoded public keys to use to authenticate signatures locally. Cannot be used with
              "jwks_url" or "oidc_discovery_url".
    oidc_discovery_ca_pem:
        description:
            - The CA certificate or chain of certificates, in PEM format, to use to validate connections to the OIDC
              Discovery URL. If not set, system certificates are used.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_oidc_auth_method_config:
        oidc_discovery_url: "https://accounts.google.com"
        oidc_client_id: "123456"
        oidc_client_secret: "123456"
        default_role: "gmail"
        verify: False
'''


def main():
    argspec = hashivault_argspec()
    argspec['bound_issuer'] = dict(required=False, type='str', default='')
    argspec['jwks_ca_pem'] = dict(required=False, type='str', default='')
    argspec['jwks_url'] = dict(required=False, type='str')
    argspec['jwt_supported_algs'] = dict(required=False, type='list', default=[])
    argspec['jwt_validation_pubkeys'] = dict(required=False, type='list', default=[])
    argspec['oidc_discovery_ca_pem'] = dict(required=False, type='str', default='')
    argspec['mount_point'] = dict(required=False, type='str', default='oidc')
    argspec['oidc_discovery_url'] = dict(required=False, type='str')
    argspec['oidc_client_id'] = dict(required=False, type='str')
    argspec['oidc_client_secret'] = dict(required=False, type='str')
    argspec['default_role'] = dict(required=False, type='str')
    required_one_of = [['oidc_discovery_url', 'jwks_url']]
    module = hashivault_init(argspec, supports_check_mode=True, required_one_of=required_one_of)
    result = hashivault_oidc_auth_method_config(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_oidc_auth_method_config(module):
    params = module.params
    client = hashivault_auth_client(params)
    changed = False
    mount_point = params.get('mount_point').strip('/')
    desired_state = dict()
    current_state = dict()
    exists = False

    token = params['token']
    namespace = params['namespace']
    headers = {'X-Vault-Token': token, 'X-Vault-Namespace': namespace}
    url = params['url']
    verify = params['verify']

    if not params.get('oidc_discovery_url'):
        desired_state['oidc_discovery_url'] = ''
    else:
        desired_state['oidc_discovery_url'] = params.get('oidc_discovery_url')
    desired_state['oidc_client_id'] = params.get('oidc_client_id')
    desired_state['default_role'] = params.get('default_role')
    desired_state['bound_issuer'] = params.get('bound_issuer')
    desired_state['jwks_ca_pem'] = params.get('jwks_ca_pem')
    if not params.get('jwks_url'):
        desired_state['jwks_url'] = ''
    else:
        desired_state['jwks_url'] = params.get('jwks_url')
    desired_state['jwt_supported_algs'] = params.get('jwt_supported_algs')
    desired_state['jwt_validation_pubkeys'] = params.get('jwt_validation_pubkeys')
    desired_state['oidc_discovery_ca_pem'] = params.get('oidc_discovery_ca_pem')

    # check if any config exists
    try:
        current_state = requests.get(url + '/v1/auth/' + mount_point + '/config', verify=verify, headers=headers)
        if current_state.status_code == 404:
            changed = True
        elif current_state.status_code == 200:
            exists = True
    except Exception:
        changed = True

    # check if current config matches desired config values, if they dont match, set changed true
    if exists:
        current_state = current_state.json()['data']
        for k, v in current_state.items():
            if k in desired_state and v != desired_state[k]:
                changed = True

    desired_state['oidc_client_secret'] = params.get('oidc_client_secret')

    # if configs dont match and checkmode is off, complete the change
    if changed and not module.check_mode:
        config_status = requests.post(url + '/v1/auth/' + mount_point + '/config', verify=verify, headers=headers,
                                      json=desired_state)
        try:
            config_status.raise_for_status()
        except Exception:
            return {'failed': True, 'msg': config_status.text, 'rc': 1}
    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
from ansible.module_utils.hashivault import hashivault_client

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_generate_root
version_added: "3.14.0"
short_description: Hashicorp Vault generate_root module
description:
    - Module to (update) generate_root Hashicorp Vault.
options:
    key:
        description:
            - vault key shard.
    nonce:
        description:
            - generate_root nonce.
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_generate_root:
      key: '{{vault_unseal_key}}'
      nonce: '{{nonce}}'
'''


def main():
    argspec = hashivault_argspec()
    argspec['key'] = dict(required=False, type='str', no_log=True)
    argspec['nonce'] = dict(required=True, type='str')
    module = hashivault_init(argspec)
    result = hashivault_generate_root(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_generate_root(params):
    key = params.get('key')
    nonce = params.get('nonce')
    client = hashivault_client(params)
    return {'status': client.generate_root(key, nonce), 'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_cluster_status
version_added: "3.16.4"
short_description: Hashicorp Vault cluster status module
description:
    - Module to get cluster status of Hashicorp Vault.
options:
    standby_ok:
        description:
            - Specifies if being a standby should still return the active status code instead of the standby status code
        default: False
    method:
      description:
        - Method to use to get cluster status, supported methods are HEAD (produces 000 (empty body)) and GET (produces
          000 application/json)
      default: HEAD
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_cluster_status:
      register: 'vault_cluster_status'
    - debug: msg="Cluster is initialized: {{vault_cluster_status.status.initialized}}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['standby_ok'] = dict(required=False, type='bool', default=True)
    argspec['method'] = dict(required=False, default="HEAD")
    module = hashivault_init(argspec)
    result = hashivault_cluster_status(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_cluster_status(params):
    client = hashivault_client(params)
    response = client.sys.read_health_status(standby_ok=params.get("standby_ok"), method=params.get("method"))
    from requests.models import Response
    if isinstance(response, Response):
        try:
            status = response.json()
        except Exception:
            status = response.content
    else:
        status = response
    return {'status': status}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
import json

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_azure_secret_engine_config
version_added: "3.17.6"
short_description: Hashicorp Vault azure secret engine config
description:
    - Module to configure an azure secret engine via variables or json file
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: azure
    subscription_id:
        description:
            - azure SPN subscription id
    tenant_id:
        description:
            - azure SPN tenant id
    client_id:
        description:
            - azure SPN client id
    client_secret:
        description:
            - azure SPN client secret
    config_file:
        description:
            - alternate way to pass SPN vars. must be json object
    environment:
        description:
            - azure environment. you probably do not want to change this
        default: AzurePublicCloud
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_azure_secret_engine_config:
        subscription_id: 1234
        tenant_id: 5689-1234
        client_id: 1012-1234
        client_secret: 1314-1234

    - hashivault_azure_secret_engine_config:
        config_file: /home/drewbuntu/azure-config.json
        mount_point: azure
'''


def main():
    argspec = hashivault_argspec()
    argspec['mount_point'] = dict(required=False, type='str', default='azure')
    argspec['subscription_id'] = dict(required=False, type='str')
    argspec['tenant_id'] = dict(required=False, type='str')
    argspec['client_id'] = dict(required=False, type='str')
    argspec['client_secret'] = dict(required=False, type='str')
    argspec['environment'] = dict(required=False, type='str', default='AzurePublicCloud')
    argspec['config_file'] = dict(required=False, type='str', default=None)
    required_together = [['subscription_id', 'client_id', 'client_secret', 'tenant_id']]

    module = hashivault_init(argspec, supports_check_mode=True, required_together=required_together)
    result = hashivault_azure_secret_engine_config(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_azure_secret_engine_config(module):
    params = module.params
    client = hashivault_auth_client(params)
    changed = False
    config_file = params.get('config_file')
    mount_point = params.get('mount_point').strip('/')
    desired_state = dict()
    current_state = dict()

    # if config_file is set, set sub_id, ten_id, client_id, client_secret from file
    # else set from passed args
    if config_file:
        desired_state = json.loads(open(params.get('config_file'), 'r').read())
        if 'environment' not in desired_state:
            desired_state['environment'] = 'AzurePublicCloud'
    else:
        desired_state['tenant_id'] = params.get('tenant_id')
        desired_state['subscription_id'] = params.get('subscription_id')
        desired_state['client_id'] = params.get('client_id')
        desired_state['client_secret'] = params.get('client_secret')
        desired_state['environment'] = params.get('environment')

    # check if engine is enabled
    changed, err = check_secrets_engines(module, client)
    if err:
        return err

    # check if current config matches desired config values, if they match, set changed to false to prevent action
    try:
        current_state = client.secrets.azure.read_config()
    except Exception:
        changed = True

    for k, v in current_state.items():
        if v != desired_state[k]:
            changed = True

    # if configs dont match and checkmode is off, complete the change
    if changed and not module.check_mode:
        client.secrets.azure.configure(mount_point=mount_point, **desired_state)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import warnings

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_list
version_added: "2.9"
short_description: Hashicorp Vault list
description:
    - The M(hashivault_list) module lists keys in Hashicorp Vault.  By
      default this will list top-level keys under `/secret`, but you
      can provide an alternate location as *secret*.  This includes both
      immediate subkeys and subkey paths, like the `vault list` command.
options:
    secret:
        description:
            - 'secret path to list.  If this does not begin with a `/`
              then it is interpreted as a subpath of `/secret`.  This
              is always interpreted as a "directory": if a key `/secret/foo`
              exists, and you pass `/secret/foo` as *secret*, then the key
              itself will not be returned, but subpaths like
              `/secret/foo/bar` will.'
        default: ''
    version:
        description:
            - version of the kv engine (int)
        default: 1
    mount_point:
        description:
            - secret mount point
        default: secret
extends_documentation_fragment: hashivault
'''
RETURN = '''
---
secrets:
    description: list of secrets found, if any
    returned: success
    type: list
    sample: ["giant", "stalks/"]
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_list:
        secret: 'giant'
        version: 2
      register: 'fie'
    - debug: msg="Known secrets are {{ fie.secrets|join(', ') }}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['version'] = dict(required=False, type='int', default=1)
    argspec['mount_point'] = dict(required=False, type='str', default='secret')
    argspec['secret'] = dict(default='', type='str')
    module = hashivault_init(argspec)
    result = hashivault_list(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_list(params):
    result = {"changed": False, "rc": 0}
    client = hashivault_auth_client(params)
    version = params.get('version')
    mount_point = params.get('mount_point')
    secret = params.get('secret')

    if secret.startswith('/'):
        secret = secret.lstrip('/')
        listo = secret.split('/')
        if listo:
            mount_point = next(iter(listo), mount_point)
            secret = '/'.join(listo[1:])
    # for backwards compatibiltiy with old hashivault_list module
    if secret.startswith('metadata/'):
        version = 2
        secret = secret.lstrip('metadata/')

    response = None
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        try:
            if version == 2:
                response = client.secrets.kv.v2.list_secrets(path=secret, mount_point=mount_point)
            else:
                response = client.secrets.kv.v1.list_secrets(path=secret, mount_point=mount_point)
        except Exception as e:
            if response is None:
                response = {}
            else:
                return {'failed': True, 'msg': str(e)}
        result['secrets'] = response.get('data', {}).get('keys', [])
    return result


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import check_pki_role
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_cert_sign
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Sign CSR ( Certificate / Intermediate / Verbatim )
description:
    - This module signs a new certificate based upon the provided CSR and the supplied parameters.
options:
    csr:
        recuired: true
        description:
            - Specifies the PEM-encoded CSR.
    role:
        description:
            - Specifies the name of the role to create.
            - 'For *verbatim* type if set, the following parameters from the role will have effect: `ttl`, `max_ttl`,
              `generate_lease`, and `no_store`.'
    common_name:
        description:
            - Specifies the requested CN for the certificate. If the CN is allowed by role policy, it will be issued.
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
    type:
        type: str
        description:
            - Sign a new certificate with `certificate` based upon the provided CSR and the supplied parameters, subject
              to the restrictions contained in the role named in the endpoint. The issuing CA certificate is returned as
              well, so that only the root CA need be in a client's trust store.
            - Use `intermediate` to configure CA certificate to issue a certificate with appropriate values for
              acting as an intermediate CA. Distribution points use the values set via config/urls. Values set in the
              CSR are ignored unless use_csr_values is set to true, in which case the values from the CSR are used
              verbatim.
            - Use `verbatim` to sign a new certificate based upon the provided CSR. Values are taken verbatim from the
              CSR; the only restriction is that this endpoint will refuse to issue an intermediate CA certificate (use
              `intermediate` type for that functionality.)
        choices: ["certificate", "intermediate", "verbatim"]
        default: certificate
    extra_params:
        description:
            Extra parameters depending on the type.
        type: dict
extends_documentation_fragment:
    - hashivault
'''

EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_cert_sign:
        role: 'tester'
        common_name: 'test.example.com'
      register: cert
    - debug: msg="{{ cert }}"

'''


def main():
    argspec = hashivault_argspec()
    argspec['csr'] = dict(required=True, type='str')
    argspec['role'] = dict(required=False, type='str')
    argspec['common_name'] = dict(required=False, type='str')
    argspec['extra_params'] = dict(required=False, type='dict', default={})
    argspec['mount_point'] = dict(required=False, type='str', default='pki')
    argspec['type'] = dict(required=False, type='str', default='certificate', choices=["certificate", "intermediate",
                                                                                       "verbatim"])

    module = hashivault_init(argspec)
    result = hashivault_pki_cert_sign(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def certificate(params, mount_point, client):
    csr = params.get('csr')
    common_name = params.get('common_name')
    extra_params = params.get('extra_params')
    role = params.get('role').strip('/')

    # check if role exists
    if not check_pki_role(name=role, mount_point=mount_point, client=client):
        return {'failed': True, 'rc': 1, 'msg': 'role not found or permission denied'}

    if not common_name:
        return {'failed': True, 'rc': 1, 'msg': 'Missing required options: common_name'}

    result = {"changed": False, "rc": 0}
    try:
        result['data'] = client.secrets.pki.sign_certificate(csr=csr, name=role, mount_point=mount_point,
                                                             common_name=common_name,
                                                             extra_params=extra_params).get('data')
        result['changed'] = True
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Exception: " + str(e)
    return result


def intermediate(params, mount_point, client):
    csr = params.get('csr')
    common_name = params.get('common_name')
    extra_params = params.get('extra_params')

    if not common_name:
        return {'failed': True, 'rc': 1, 'msg': 'Missing required options: common_name'}

    result = {"changed": False, "rc": 0}
    try:
        result['data'] = client.secrets.pki.sign_intermediate(csr=csr, common_name=common_name,
                                                              extra_params=extra_params,
                                                              mount_point=mount_point).get('data')
        result['changed'] = True
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Exception: " + str(e)
    return result


def verbatim(params, mount_point, client):
    csr = params.get('csr')
    extra_params = params.get('extra_params')
    role = params.get('role').strip('/')

    # check if role exists
    if not check_pki_role(name=role, mount_point=mount_point, client=client):
        return {'failed': True, 'rc': 1, 'msg': 'role not found or permission denied'}

    result = {"changed": False, "rc": 0}
    try:
        result['data'] = client.secrets.pki.sign_verbatim(csr=csr, name=role, extra_params=extra_params,
                                                          mount_point=mount_point).get('data')
        result['changed'] = True
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Exception: " + str(e)
    return result


@hashiwrapper
def hashivault_pki_cert_sign(module):
    supported_types = {
        'certificate': certificate,
        'intermediate': intermediate,
        'verbatim': verbatim
    }
    params = module.params
    client = hashivault_auth_client(params)
    mount_point = params.get('mount_point').strip('/')

    # check if engine is enabled
    _, err = check_secrets_engines(module, client)
    if err:
        return err
    return supported_types[params.get('type')](params=params, mount_point=mount_point, client=client)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_userpass
version_added: "3.12.0"
short_description: Hashicorp Vault userpass user management module
description:
    - Module to manage userpass users in Hashicorp Vault.
options:
    name:
        description:
            - user name to create.
    pass:
        description:
            - user to create password.
    pass_update:
        description:
            - whether to update the password if user exists
        default: False
    policies:
        description:
            - user policies.
        default: default
    token_bound_cidrs:
        description:
            - List of CIDR blocks; if set, specifies blocks of IP addresses which can authenticate successfully, and
              ties the resulting token to these blocks as well.
    state:
        description:
            - whether create/update or delete the user
    mount_point:
        description:
            - default The "path" (app-id) the auth backend is mounted on.
        default: userpass
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_userpass_create:
      name: 'bob'
      pass: 'S3cre7s'
      policies: 'bob'
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['pass'] = dict(required=False, type='str', default=None, no_log=True)
    argspec['pass_update'] = dict(required=False, type='bool', default=False)
    argspec['policies'] = dict(required=False, type='list', default=[])
    argspec['token_bound_cidrs'] = dict(required=False, type='list', default=[])
    argspec['state'] = dict(required=False, choices=['present', 'absent'], default='present')
    argspec['mount_point'] = dict(required=False, type='str', default='userpass')
    module = hashivault_init(argspec)
    result = hashivault_userpass(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def hashivault_userpass_update(client, user_details, user_name, user_pass, user_pass_update, user_policies,
                               mount_point, token_bound_cidrs):
    policies_changed = False
    token_bound_cidrs_changed = False
    password_change_allowed = user_pass_update and user_pass

    if set(user_details['data'].get('policies', [])) != set(user_policies):
        policies_changed = True

    if set(user_details['data']['token_bound_cidrs']) != set(token_bound_cidrs):
        token_bound_cidrs_changed = True

    attribute_changed = policies_changed or token_bound_cidrs_changed

    if password_change_allowed and not attribute_changed:
        client.update_userpass_password(user_name, user_pass, mount_point=mount_point)
        return {'changed': True}

    if password_change_allowed and attribute_changed:
        client.create_userpass(user_name, user_pass, user_policies, mount_point=mount_point,
                               token_bound_cidrs=token_bound_cidrs)
        return {'changed': True}

    if not password_change_allowed and attribute_changed:
        if token_bound_cidrs_changed:
            err_msg = u"token_bound_cidrs can only be changed if user_pass is specified and user_pass_update is True"
            return {
                'rc': 1,
                'failed': True,
                'msg': err_msg
            }

        if policies_changed:
            client.update_userpass_policies(user_name, user_policies, mount_point=mount_point)
            return {'changed': True}

    return {'changed': False}


@hashiwrapper
def hashivault_userpass(params):
    client = hashivault_auth_client(params)
    state = params.get('state')
    name = params.get('name')
    password = params.get('pass')
    password_update = params.get('pass_update')
    policies = params.get('policies')
    token_bound_cidrs = params.get('token_bound_cidrs')
    mount_point = params.get('mount_point')
    if state == 'present':
        try:
            user_details = client.read_userpass(name, mount_point=mount_point)
        except Exception:
            if password is not None:
                client.create_userpass(name, password, policies, token_bound_cidrs=token_bound_cidrs,
                                       mount_point=mount_point)
                return {'changed': True}
            else:
                return {'failed': True, 'msg': 'pass must be provided for new users'}
        else:
            return hashivault_userpass_update(client, user_details, user_name=name, user_pass=password,
                                              user_pass_update=password_update, user_policies=policies,
                                              mount_point=mount_point, token_bound_cidrs=token_bound_cidrs)
    elif state == 'absent':
        try:
            client.read_userpass(name, mount_point=mount_point)
        except Exception:
            return {'changed': False}
        else:
            client.delete_userpass(name, mount_point=mount_point)
            return {'changed': True}
    else:
        return {'failed': True, 'msg': 'Unkown state type'}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_ca
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Generate Root/Intermediate
description:
    - 'WARNING: if king is `intermediate` and signed CSR have not been inported back to vault, module will regenerate
      private key and create new CSR.'
    - This module generates a new private key and a CSR for signing or a new self-signed CA certificate and private key.
    - If using Vault as a root, and for many other CAs, the various parameters on the final certificate are set at
      signing time and may or may not honor the parameters set here.
    - This will overwrite any previously existing CA private key.
    - This is mostly meant as a helper function, and not all possible parameters that can be set in a CSR are supported.
options:
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
    type:
        type: str
        choices: ["exported", "internal"]
        default: internal
        description:
            - Specifies the type of the root to create, If `exported`, the private key will be returned in the
              response;
            - If it is `internal` the private key will not be returned and cannot be retrieved later.
    common_name:
        type: str
        description:
            - Specifies the requested CN for the certificate. [required]
    kind:
        type: str
        choices: ["root", "intermediate"]
        default: root
        description:
            - Specifies the kind of CA certificate.
    state:
        type: str
        choices: ["present", "absent"]
        default: present
        description:
            - Do you want for this config to be present or absent
    config:
        description:
            - Collection of properties for pki generate root. Ref.
              U(https://www.vaultproject.io/api-docs/secret/pki#generate-root)
        type: dict
        suboptions:
            alt_names:
                type: str
                description:
                    - Specifies the requested Subject Alternative Names, in a comma-delimited list.
                    - These can be host names or email addresses; they will be parsed into their respective fields.
            ip_sans:
                type: str
                description:
                    - Specifies the requested IP Subject Alternative Names, in a comma-delimited list.
            uri_sans:
                type: str
                description:
                    - Specifies the requested URI Subject Alternative Names, in a comma-delimited list.
            other_sans:
                type: str
                description:
                    - Specifies custom OID/UTF8-string SANs.
                    - These must match values specified on the role in `allowed_other_sans` (see role creation for
                      allowed_other_sans globbing rules).
                    - The format is the same as OpenSSL `<oid>;<type>:<value>` where the only current valid type is
                      `UTF8`
                    - This can be a comma-delimited list or a JSON string slice.
            ttl:
                type: str
                description:
                    - Specifies the requested Time To Live (after which the certificate will be expired).
                    - This cannot be larger than the engine's max (or, if not set, the system max).
            format:
                type: str
                default: "pem"
                choice: ["pem", "der", "pem_bundle"]
                description:
                    - Specifies the format for returned data.
                    - If `der`, the output is base64 encoded.
                    - If `pem_bundle`, the `certificate` field will contain the private key (if exported) and
                      certificate, concatenated;
                    - if the issuing CA is not a Vault-derived self-signed root, this will be included as well.
            private_key_format:
                type: str
                default: der
                description:
                    - Specifies the format for marshaling the private key.
                    - Defaults to `der` which will return either base64-encoded DER or PEM-encoded DER, depending on
                      the value of `format`.
                    - The other option is `pkcs8` which will return the key marshalled as PEM-encoded PKCS8
            key_type:
                type: str
                default: "rsa"
                choice: ["rsa", "ec"]
                description:
                    - Specifies the desired key type.
            key_bits:
                type: int
                default: 2048
                description:
                    - Specifies the number of bits to use
            max_path_length:
                type: int
                default: -1
                description:
                    - Specifies the maximum path length to encode in the generated certificate.
                    - A limit of `-1` means no limit.
                    - Unless the signing certificate has a maximum path length set, in which case the path length is set
                      to one less than that of the signing certificate.
                    - A limit of `0` means a literal path length of zero.
            exclude_cn_from_sans:
                type: bool
                default: false
                description:
                    - If set, the given `common_name` will not be included in DNS or Email Subject Alternate Names (as
                      appropriate).
                    - Useful if the CN is not a hostname or email address, but is instead some human-readable
                      identifier.
            permitted_dns_domains:
                type: list
                description:
                    - A list containing DNS domains for which certificates are allowed to be issued or signed by this CA
                      certificate.
                    - Note that subdomains are allowed, as per U(https://tools.ietf.org/html/rfc5280#section-4.2.1.10).
            ou:
                type: list
                description:
                    - Specifies the `OU` (OrganizationalUnit) values in the subject field of the resulting certificate.
            organization:
                type: list
                description:
                    - Specifies the `O` (Organization) values in the subject field of the resulting certificate.
            country:
                type: list
                description:
                    - Specifies the `C` (Country) values in the subject field of the resulting certificate.
            locality:
                type: list
                description:
                    - Specifies the `L` (Locality) values in the subject field of the resulting certificate.
            province:
                type: list
                description:
                    - Specifies the `ST` (Province) values in the subject field of the resulting certificate.
            street_address:
                type: list
                description:
                    - Specifies the Street Address values in the subject field of the resulting certificate.
            postal_code:
                type: list
                description:
                    - Specifies the Postal Code values in the subject field of the resulting certificate.
            serial_number:
                type: string
                description:
                    - Specifies the Serial Number, if any.
                    - Otherwise Vault will generate a random serial for you.
                    - If you want more than one, specify alternative names in the alt_names map using OID 2.5.4.5.
extends_documentation_fragment:
    - hashivault
'''
EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - name: Delete Root
      hashivault_pki_ca:
        common_name: my common name
'''


def main():
    argspec = hashivault_argspec()
    argspec['common_name'] = dict(required=True, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='pki')
    argspec['state'] = dict(required=False, type='str', default='present', choices=['present', 'absent'])
    argspec['kind'] = dict(required=False, type='str', default='root', choices=['root', 'intermediate'])
    argspec['type'] = dict(required=False, type='str', default='internal', choices=['internal', 'exported'])
    argspec['config'] = dict(required=False, type='dict', default={})

    supports_check_mode = True

    module = hashivault_init(argspec, supports_check_mode)
    result = hashivault_pki_ca(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_ca(module):
    params = module.params
    client = hashivault_auth_client(params)

    common_name = params.get('common_name')
    mount_point = params.get('mount_point').strip('/')
    state = params.get('state')
    kind = params.get('kind')
    type = params.get('type')
    config = params.get('config')

    exists = False

    # check if engine is enabled
    changed, err = check_secrets_engines(module, client)
    if err:
        return err

    # check if CA certificate exists
    if client.secrets.pki.read_ca_certificate(mount_point=mount_point):
        # WARNING: if king is `intermediate` and signed CSR have not been inported back to vault, module will regenerate
        # private key and create new CSR. This check will not see that CA exist although private key is generate, and
        # CSR might be in proccess to be signed.
        exists = True

    if (exists and state == 'absent') or (not exists and state == 'present'):
        changed = True

    result = {"changed": changed, "rc": 0}

    # make the changes!
    if changed and state == 'present' and not module.check_mode:
        if kind == 'root':
            resp = client.secrets.pki.generate_root(type=type, common_name=common_name, extra_params=config,
                                                    mount_point=mount_point)
            result['data'] = resp.get('data')
            if resp.get('warnings'):
                result['warnings'] = resp.get('warnings')
        elif kind == 'intermediate':
            resp = client.secrets.pki.generate_intermediate(type=type, common_name=common_name, extra_params=config,
                                                            mount_point=mount_point)
            result['data'] = resp.get('data')
            if resp.get('warnings'):
                result['warnings'] = resp.get('warnings')

    elif changed and state == 'absent' and not module.check_mode:
        client.secrets.pki.delete_root(mount_point=mount_point)

    return result


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import check_pki_role
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_cert_issue
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Generate Certificate
description:
    - This module generates a new set of credentials (private key and certificate) based on the role named in the
      module.
    - The issuing CA certificate is returned as well, so that only the root CA need be in a client's trust store.
options:
    role:
        recuired: true
        description:
            - Specifies the name of the role to create.
    common_name:
        recuired: true
        description:
            - Specifies the requested CN for the certificate. If the CN is allowed by role policy, it will be issued.
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
    extra_params:
        description:
            - "Collection of properties from pki role U(https://www.vaultproject.io/api-docs/secret/pki#parameters-6)"
        type: dict
        suboptions:
        alt_names:
            type: string
            description:
                - Specifies requested Subject Alternative Names, in a comma-delimited list. These can be host names or
                  email addresses; they will be parsed into their respective fields. If any requested names do not match
                  role policy, the entire request will be denied.
        ip_sans:
            type: string
            description:
                - Specifies requested IP Subject Alternative Names, in a comma-delimited list. Only valid if the role
                  allows IP SANs (which is the default).
        uri_sans:
            type: string
            description:
                - Specifies the requested URI Subject Alternative Names, in a comma-delimited list.
        other_sans:
            type: string
            description:
                - Specifies custom OID/UTF8-string SANs. These must match values specified on the role in
                  allowed_other_sans (see role creation for allowed_other_sans globbing rules). The format is the same
                  as OpenSSL <oid>;<type>:<value> where the only current valid type is UTF8. This can be a
                  comma-delimited list or a JSON string slice.
        ttl:
            type: string
            description:
                - Specifies requested Time To Live. Cannot be greater than the role's max_ttl value. If not provided,
                  the role's ttl value will be used. Note that the role values default to system values if not
                  explicitly set.
        format:
            type: string
            description:
                - Specifies the format for returned data. Can be pem, der, or pem_bundle; defaults to pem. If der, the
                  output is base64 encoded. If pem_bundle, the certificate field will contain the private key and
                  certificate, concatenated; if the issuing CA is not a Vault-derived self-signed root, this will be
                  included as well.
        private_key_format:
            type: string
            description:
                - Specifies the format for marshaling the private key. Defaults to der which will return either
                  base64-encoded DER or PEM-encoded DER, depending on the value of format. The other option is pkcs8
                  which will return the key marshalled as PEM-encoded PKCS8.
        exclude_cn_from_sans:
            type: bool
            description:
                - If true, the given common_name will not be included in DNS or Email Subject Alternate Names
                  (as appropriate). Useful if the CN is not a hostname or email address, but is instead some
                  human-readable identifier.
extends_documentation_fragment:
    - hashivault
'''

EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_cert_issue:
        role: 'tester'
        common_name: 'test.example.com'
      register: cert
    - debug: msg="{{ cert }}"

'''


def main():
    argspec = hashivault_argspec()
    argspec['role'] = dict(required=True, type='str')
    argspec['common_name'] = dict(required=True, type='str')
    argspec['extra_params'] = dict(required=False, type='dict', default={})
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    module = hashivault_init(argspec)
    result = hashivault_pki_cert_issue(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_cert_issue(module):
    params = module.params
    client = hashivault_auth_client(params)

    role = params.get('role').strip('/')
    common_name = params.get('common_name')
    extra_params = params.get('extra_params')
    mount_point = params.get('mount_point').strip('/')

    # check if engine is enabled
    _, err = check_secrets_engines(module, client)
    if err:
        return err

    if not check_pki_role(name=role, mount_point=mount_point, client=client):
        return {'failed': True, 'rc': 1, 'msg': 'role not found or permission denied'}

    result = {"changed": False, "rc": 0}
    try:
        result['data'] = client.secrets.pki.generate_certificate(name=role, common_name=common_name,
                                                                 extra_params=extra_params,
                                                                 mount_point=mount_point).get('data')
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Exception: " + str(e)
    return result


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper
import json
from ast import literal_eval

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_azure_secret_engine_role
version_added: "3.17.6"
short_description: Hashicorp Vault azure secret engine role
description:
    - Module to define a Azure role that vault can generate dynamic credentials for vault
options:
    mount_point:
        description:
            - name of the secret engine mount name.
        default: azure
    name:
        description:
            - name of the role in vault
    azure_role:
        description:
            - list of nested dicts filled with role content [{"role_name":"", "scope":""}]
    azure_role_file:
        description:
            - file with a single dict, azure_role
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_azure_secret_engine_role:
        name: contributor-role
        azure_role: [{ "role_name": "Contributor","scope": "/subscriptions/sub1234"}]

    - hashivault_azure_secret_engine_role:
        name: contributor-role
        azure_role_file: /users/dmullen/my-role-file.json
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['azure_role'] = dict(required=False, type='str')
    argspec['azure_role_file'] = dict(required=False, type='str')
    argspec['mount_point'] = dict(required=False, type='str', default='azure')
    mutually_exclusive = [['azure_role', 'azure_role_file']]

    module = hashivault_init(argspec, supports_check_mode=True, mutually_exclusive=mutually_exclusive)
    result = hashivault_azure_secret_engine_role(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_azure_secret_engine_role(module):
    params = module.params
    client = hashivault_auth_client(params)
    azure_role_file = params.get('azure_role_file')
    mount_point = params.get('mount_point').strip('/')
    azure_role = params.get('azure_role')
    name = params.get('name').strip('/')
    changed = False

    # if azure_role_file is set, set azure_role to contents
    # else assume azure_role is set and use that value
    if azure_role_file:
        azure_role = json.loads(open(params.get('azure_role_file'), 'r').read())['azure_role']

    # check if engine is enabled
    changed, err = check_secrets_engines(module, client)
    if err:
        return err

    # check if role exists or any at all
    try:
        existing_roles = client.secrets.azure.list_roles(mount_point=mount_point)
        if name not in existing_roles['keys']:
            changed = True
    except Exception:
        changed = True

    # azure_role comes from json which is assigned as a str object type, convert to py objs
    azure_role = literal_eval(azure_role)
    if not changed:
        # check if role content == desired
        current = client.secrets.aws.read_role(name=name, mount_point=mount_point)['data']['azure_roles']
        caught = 0
        for i in azure_role:
            for i2 in current:
                if i.items() <= i2.items():
                    caught = caught + 1
        if caught != len(azure_role) or caught != len(current):
            changed = True

    # make the changes!
    if changed and not module.check_mode:
        client.secrets.azure.create_or_update_role(name=name, azure_roles=azure_role)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
import warnings

from hvac.exceptions import InvalidPath

from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_write
version_added: "0.1"
short_description: Hashicorp Vault write module
description:
    - Module to write to Hashicorp Vault.
options:
    version:
        description:
            - version of the kv engine (int)
        default: 1
    mount_point:
        description:
            - secret mount point
        default: secret
    secret:
        description:
            - vault secret to write.
    data:
        description:
            - Keys and values to write.
    alternate_data:
        description:
            - Keys and values to write. Use this if you need the returned data.
    update:
        description:
            - Update rather than overwrite.
        default: False
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_write:
        secret: giant
        data:
            foo: foe
            fie: fum
'''


def main():
    argspec = hashivault_argspec()
    argspec['version'] = dict(required=False, type='int', default=1)
    argspec['mount_point'] = dict(required=False, type='str', default='secret')
    argspec['secret'] = dict(required=True, type='str')
    argspec['update'] = dict(required=False, default=False, type='bool')
    argspec['data'] = dict(required=False, default={}, type='dict', no_log=True)
    argspec['alternate_data'] = dict(required=False, default={}, type='dict')
    module = hashivault_init(argspec, supports_check_mode=True)
    result = hashivault_write(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


def _convert_to_seconds(original_value):
    try:
        value = str(original_value)
        seconds = 0
        if 'h' in value:
            ray = value.split('h')
            seconds = int(ray.pop(0)) * 3600
            value = ''.join(ray)
        if 'm' in value:
            ray = value.split('m')
            seconds += int(ray.pop(0)) * 60
            value = ''.join(ray)
        if value:
            ray = value.split('s')
            seconds += int(ray.pop(0))
        return seconds
    except Exception:
        pass
    return original_value


def hashivault_changed(old_data, new_data):
    if sorted(old_data.keys()) != sorted(new_data.keys()):
        return True
    for key in old_data:
        old_value = old_data[key]
        new_value = new_data[key]
        if old_value == new_value:
            continue
        if key != 'ttl' and key != 'max_ttl':
            return True
        old_value = _convert_to_seconds(old_value)
        new_value = _convert_to_seconds(new_value)
        if old_value != new_value:
            return True
    return False


@hashiwrapper
def hashivault_write(module):
    result = {"changed": False, "rc": 0}
    params = module.params
    client = hashivault_auth_client(params)
    version = params.get('version')
    mount_point = params.get('mount_point')
    secret = params.get('secret')
    data = params.get('data')
    data = data or params.get('alternate_data')

    if secret.startswith('/'):
        secret = secret.lstrip('/')
        mount_point = ''
    if mount_point:
        secret_path = '%s/%s' % (mount_point, secret)
    else:
        secret_path = secret

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        changed = True
        write_data = data

        if params.get('update') or module.check_mode:
            # Do not move these reads outside of the update
            read_data = None
            try:
                if version == 2:
                    read_data = client.secrets.kv.v2.read_secret_version(secret, mount_point=mount_point)
                else:
                    read_data = client.read(secret_path) or {}
            except InvalidPath:
                read_data = None
            except Exception as e:
                result['rc'] = 1
                result['failed'] = True
                error_string = "%s(%s)" % (e.__class__.__name__, e)
                result['msg'] = u"Error %s reading %s" % (error_string, secret_path)
                return result
            if not read_data:
                read_data = {}
            read_data = read_data.get('data', {})

            if version == 2:
                read_data = read_data.get('data', {})

            write_data = dict(read_data)
            write_data.update(data)

            # result['write_data'] = write_data
            # result['read_data'] = read_data
            changed = hashivault_changed(read_data, write_data)

        if changed:
            if not module.check_mode:
                try:
                    if version == 2:
                        returned_data = client.secrets.kv.v2.create_or_update_secret(mount_point=mount_point,
                                                                                     path=secret, secret=write_data)
                    else:
                        returned_data = client.write(secret_path, **write_data)
                    if returned_data:
                        from requests.models import Response
                        if isinstance(returned_data, Response):
                            result['data'] = returned_data.text
                        else:
                            result['data'] = returned_data
                except Exception as e:
                    result['rc'] = 1
                    result['failed'] = True
                    error_string = "%s(%s)" % (e.__class__.__name__, e)
                    result['msg'] = u"Error %s writing %s" % (error_string, secret_path)
                    return result

            result['msg'] = u"Secret %s written" % secret_path
        result['changed'] = changed
    return result


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_cert_get
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Read Certificate
description:
    - This module retrieves one of a selection of certificates.
options:
    serial:
        recuired: true
        defaul: ca
        choices: ["ca", "ca_chain", "crl", "<serial>"]
        description:
            - Specifies the serial of the key to read.
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
extends_documentation_fragment:
    - hashivault
'''
EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_cert_get:
      register: cert
    - debug: msg="{{ cert }}"

    - hashivault_pki_cert_get:
        serial: 0f-d0-bf-cc-7a-d2-b5-b8-ec-6d-8f-cb-9d-0a-c7-d1-e2-3b-82-7d
      register: cert
    - debug: msg="{{ cert }}"
'''


def main():
    argspec = hashivault_argspec()
    argspec['serial'] = dict(required=False, type='str', default='ca')
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    module = hashivault_init(argspec)
    result = hashivault_pki_cert_get(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_cert_get(module):
    params = module.params
    client = hashivault_auth_client(params)

    serial = params.get('serial')
    mount_point = params.get('mount_point').strip('/')

    # check if engine is enabled
    _, err = check_secrets_engines(module, client)
    if err:
        return err

    try:
        return {'data': client.secrets.pki.read_certificate(serial=serial, mount_point=mount_point).get('data')}
    except Exception:
        return {'data': {}}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_read_to_file
version_added: "3.8.3"
short_description: Hashicorp Vault read module
description:
    - "Reads and deocdes a base64 encoded file from Hashicorp Vault and saves it to disk. Implementation in\
     `/plugins/action/hashivault_read_to_file.py`."
options:
    secret:
        description:
            - vault secret to read.
    key:
        description:
            - secret key/name of file to read from vault.
    dest:
        description:
            - fully qualified path name of file to write to remote host.
    force:
        description:
            - force overwrite of file.
        default: false
    mode:
        description:
            - file permissions of file to write on remote host.
            - in octal, don't forget leading zero!
        default: 0664
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_read_to_file:
        secret: 'giant'
        key: 'foo.dat'
        dest: '/tmp/foo.dat'
'''



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

DEFAULT_TTL = 2764800
ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_namespace
version_added: "4.0.1"
short_description: Hashicorp Vault create / delete namespaces
description:
    - Module to create or delete Hashicorp Vault namespaces (enterprise only)
options:
    name:
        description:
            - name of the namespace
    state:
        description:
            - state of secret backend. choices: present, disabled
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_namespace:
        name: teama

    - name: "create a child namespace 'team1' in 'teama' ns: teama/team1"
      hashivault_namespace:
        name: team1
        namespace: teama
'''


def main():
    argspec = hashivault_argspec()
    argspec['name'] = dict(required=True, type='str')
    argspec['state'] = dict(required=False, type='str', choices=['present', 'absent'], default='present')
    module = hashivault_init(argspec)
    result = hashivault_secret_engine(module)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_secret_engine(module):
    params = module.params
    client = hashivault_auth_client(params)
    name = params.get('name')
    state = params.get('state')
    current_state = dict()
    exists = False
    changed = False

    try:
        # does the ns exist already?
        current_state = client.sys.list_namespaces()['data']['keys']
        if (name + '/') in current_state:
            exists = True
    except Exception:
        # doesnt exist
        pass

    # doesnt exist and should or does exist and shouldnt
    if (exists and state == 'absent') or (not exists and state == 'present'):
        changed = True

    # create
    if changed and not exists and state == 'present' and not module.check_mode:
        client.sys.create_namespace(path=name)

    # delete
    elif changed and (state == 'absent' or state == 'disabled') and not module.check_mode:
        client.sys.delete_namespace(path=name)

    return {'changed': changed}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['stableinterface'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = '''
---
module: hashivault_audit
version_added: "2.2.0"
short_description: Hashicorp Vault audit module
description:
    - Module to enable/disable audit backends in Hashicorp Vault.
options:
    device_type:
        description:
            - device_type of auditor
    path:
        description:
            - path of auditor
        default: value of device_type
    description:
        description:
            - description of auditor
    options:
        description:
            - options for auditor
    state:
        description:
            - should auth mount be enabled or disabled
        default: enabled
extends_documentation_fragment: hashivault
'''
EXAMPLES = '''
---
- hosts: localhost
  tasks:
    - hashivault_audit:
        device_type: syslog
'''


def main():
    argspec = hashivault_argspec()
    argspec['device_type'] = dict(required=True, type='str')
    argspec['description'] = dict(required=False, type='str', default='')
    argspec['options'] = dict(required=False, type='dict')
    argspec['path'] = dict(required=False, type='str')
    argspec['state'] = dict(required=False, type='str', default='enabled',
                            choices=['enabled', 'enable', 'present', 'disabled', 'disable', 'absent'])
    module = hashivault_init(argspec)
    result = hashivault_audit(module.params)
    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_audit(params):
    client = hashivault_auth_client(params)
    device_type = params.get('device_type')
    description = params.get('description')
    options = params.get('options')
    path = params.get('path')
    state = params.get('state')
    if state in ['enabled', 'enable', 'present']:
        state = 'enabled'
    else:
        state = 'disabled'

    result = client.sys.list_enabled_audit_devices()
    backends = result.get('data', result)
    if not path:
        path = device_type + "/"
    if path[-1] != "/":
        path = path + "/"

    if state == 'enabled':
        if path in backends:
            return {'changed': False, 'msg': 'Backend exists and Vault does not support update'}
        client.sys.enable_audit_device(device_type=device_type, description=description, options=options, path=path)
    elif state == 'disabled':
        if path not in backends:
            return {'changed': False, 'msg': 'Backend does not exist'}
        client.sys.disable_audit_device(path=path)
    return {'changed': True}


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from ansible.module_utils.hashivault import check_secrets_engines
from ansible.module_utils.hashivault import hashivault_auth_client
from ansible.module_utils.hashivault import hashivault_argspec
from ansible.module_utils.hashivault import hashivault_init
from ansible.module_utils.hashivault import hashiwrapper

ANSIBLE_METADATA = {'status': ['preview'], 'supported_by': 'community', 'version': '1.1'}
DOCUMENTATION = r'''
---
module: hashivault_pki_tidy
version_added: "4.5.0"
short_description: Hashicorp Vault PKI Tidy
description:
    - This endpoint retrieves one of a selection of certificates.
options:
    mount_point:
        default: pki
        description:
            - location where secrets engine is mounted. also known as path
    config:
        type: dict
        description:
            - Collection of properties for pki tidy endpoint. Ref.
              U(https://www.vaultproject.io/api-docs/secret/pki#tidy)
        suboptions:
            tidy_cert_store:
                type: bool
                default: false
                description:
                    - Specifies whether to tidy up the certificate store.
            tidy_revoked_certs:
                type: bool
                default: false
                description:
                    - Set to true to expire all revoked and expired certificates, removing them both from the CRL and
                      from storage.
                    - The CRL will be rotated if this causes any values to be removed.
            safety_buffer:
                type: str
                default: 72h
                description:
                    - Specifies A duration used as a safety buffer to ensure certificates are not expunged prematurely;
                    - as an example, this can keep certificates from being removed from the CRL that, due to clock skew,
                      might still be considered valid on other hosts.
                    - For a certificate to be expunged, the time must be after the expiration time of the certificate
                      (according to the local clock) plus the duration of safety_buffer.
extends_documentation_fragment:
    - hashivault
'''

EXAMPLES = r'''
---
- hosts: localhost
  tasks:
    - hashivault_pki_tidy:
        safety_buffer: 96h
        tidy_revoked_certs: true
'''


def main():
    argspec = hashivault_argspec()
    argspec['config'] = dict(required=False, type='dict', default={})
    argspec['mount_point'] = dict(required=False, type='str', default='pki')

    module = hashivault_init(argspec)
    result = hashivault_pki_tidy(module)

    if result.get('failed'):
        module.fail_json(**result)
    else:
        module.exit_json(**result)


@hashiwrapper
def hashivault_pki_tidy(module):
    params = module.params
    client = hashivault_auth_client(params)

    config = params.get('config')
    mount_point = params.get('mount_point').strip('/')

    # check if engine is enabled
    _, err = check_secrets_engines(module, client)
    if err:
        return err

    result = {"changed": False, "rc": 0}
    try:
        client.secrets.pki.tidy(extra_params=config, mount_point=mount_point)
    except Exception as e:
        result['rc'] = 1
        result['failed'] = True
        result['msg'] = u"Exception: " + str(e)
    return result


if __name__ == '__main__':
    main()
